--[[ 
Functions that extend local sqlite database access to allow for simple modifications of a key-value store.

Any changes made to this file should be made to the same file on all platforms.

Java clients note: as of 5/19/11 this file is not used, since on android the __index of a class's metatable is a function rather than a table, therefore we cannot add to it in lua.  On BB we get a weird error when we try to call a function defined in lua for a class's metatable.
These functions are implemented in java there, and are included here for reference until we can
figure out how to include this file on all platforms.
]]--

do
    local tableName = 'NameValueTable'
    local keyColumn = 'PropertyName'
    local valueColumn = 'PropertyValue'
    local createdColumn = 'CreatedDtm'
    local updatedColumn = 'UpdatedDtm'
    
    local mt = getmetatable(propertystore)
    local propertyfuncs = mt.__index
    
    -- Check if the given key exists in the table; insert the key/value if it doesn't and update the existing key if it does.
    function propertyfuncs:setValue(key, value)
        local createQuery = string.format("CREATE TABLE IF NOT EXISTS %s (%s TEXT PRIMARY KEY, %s TEXT, %s TEXT CONSTRAINT default_create DEFAULT CURRENT_TIMESTAMP, %s TEXT CONSTRAINT default_update DEFAULT CURRENT_TIMESTAMP)", tableName, keyColumn, valueColumn, createdColumn, updatedColumn)

        local success, reason = self:executeQuery(createQuery)
        if not success then
            -- if we can't create the table for some strange reason, we should notify the caller.
            return success, reason 
        end
    	
        local keyExists = self:scalarQuery(string.format("SELECT count(*) from %s where %s='%s'", tableName, keyColumn, key)) > 0
        local storeQuery = ''
        if keyExists then
            storeQuery = string.format("UPDATE %s set %s='%s', %s=CURRENT_TIMESTAMP where %s='%s'", tableName, valueColumn, value, updatedColumn, keyColumn, key)
        else
            storeQuery = string.format("INSERT INTO %s (%s, %s) VALUES ('%s','%s')", tableName, keyColumn, valueColumn, key, value)
        end
        return self:executeQuery(storeQuery)
    end
    
    -- Deletes the record corresponding to the given key from the table.
    function propertyfuncs:clearValue(key)
        local clearQuery = string.format("DELETE FROM %s where %s='%s'", tableName, keyColumn, key)
        return self:executeQuery(clearQuery)
    end
    
    -- Deletes all records from the property table.
    function propertyfuncs:clearAllValues()
        local clearQuery = string.format("DELETE FROM %s", tableName)
        return self:executeQuery(clearQuery)
    end
    
    -- gets the value of the given key from the table, or nil if there is no value for the given key.
    function propertyfuncs:getValue(key)
        local getQuery = string.format("SELECT %s from %s where %s='%s'", valueColumn, tableName, keyColumn, key)
        return self:scalarQuery(getQuery)
    end
end

